(* ::Package:: *)

(* 
 *
 * mConnect - Connect MATLAB from within Mathematica
 *
 * Author: Robert, Jan Lellmann (Jan.Lellmann@web.de)
 *
 * Based on mEngine 0.61 (Robert, zhangchenhui99@mails.tsinghua.edu.cn)
 *
 *)

BeginPackage["mConnect`"]

mShow::usage=	
	"mShow[] shows the MATLAB command window."

mHide::usage=
	"mHide[] hides the MATLAB command window."

mCmd::usage=
	"mCmd[\"command\"] executes command in MATLAB, and returns the output string.\n" <>
	"mCmd[\"string1\",\"string2\",...] joins all strings (without spaces) and executes the resulting command.\n"<>
	"mCmd[\"command(\",var,\"); ...\"] transfers var into the Matlab workspace under a temporary id and executes the resulting command with var replaced by the generated id.\n"<>
	"mCmd[...,Debug \[Rule] True] writes out data and command to \"debug_data.mat\" and \"debug_code.m\" prior to execution.\n"<>
	"mCmd[...,Debug \[Rule] True, DebugFile \[Rule] \"file\"] replaces \"debug\" by \"file\" in the above example.";

mCmd::arg=
	"The argument to mCmd should be a string."

mGetCmdList::usage=
	"mGetCmdList[n,commands,...] executes command in MATLAB and returns multiple results as a list. The console output of the command is discarded.\n
	n is the number of output arguments expected from the command. mGetCmdList supports the same command syntax and options as mCmd.";

mGetCmdList::arg=
	"The command argument to mGetCmdList must be a string and n must be a nonnegative integer."

mGetCmd::usage=
	"mGetCmdList[commands,...] executes command in MATLAB and returns a single result. The console output of the command is discarded.\n
    The command is expected to provide a single output argument. mGetCmdList supports the same command syntax and options as mCmd.";

mGetCmd::arg=
	"The command argument to mGetCmd must be a string."

mPut::usage=
	"mPut[\"x\"] puts variable with name x into the MATLAB workspace. Note that MATLAB automatically removes any trailing singleton (1-element) dimensions for array with more than two dimensions.\n"<>
    "mPut[\"x\",\"v\"] puts variable v into the MATLAB workspace with name x";

mPut::arg=
	"The argument to mPut should either be a string corresponding to a variable name, 
	a list of such strings, or a string (MATLAB variable name) and an expression (source variable)."

mGet::usage=
	"mGet[\"x\"] gets variable x from the MATLAB workspace. Dimension[] will be equal to size(f), with trailing singleton (1-element) dimensions removed."

mGet::arg=
	"The argument to mGet should either be a string corresponding to a variable name, 
	or a list of such strings."

mVar::usage = "mVar is a placeholder for Mathematica variables in Matlab code."

mGetLastError::usage =
	"Returns a formatted message with details about the last error that occured in the MATLAB session. If there was no error, an empty string is returned.";
mGetLastError::arg =
	"";

mClearLastError::usage =
	"Clear the last error. mGetLastError will return an empty string afterwards.";
mClearLastError::arg =
	"";

mInterpretString::usage = "Convert special characters represented as escape sequences \\[...] to the actual characters."
mInterpretString::arg = "The argument should be a string obtained from mGet or mCmd"

mGetClass::usage = "Returns MATLAB class id in string representation for a given variable name."
mGetClass::arg = "The argument should be a string with the MATLAB variable name."

mCmd::matlaberror=
	"MATLAB command failed, errors:";

mCmd::matlaberroroutput=
	"Full output of failed MATLAB command:";

SqueezeTrailing::usage=
	"SqueezeTrailing[\"x\"] removes trailing singleton (1-element) dimensions.";

(* This imports all the eng functions *)
eng = Install["mConnect-"<>$SystemID<>"-"<>$ProcessorType<>".mathlink"]

Begin["`Private`"]

ToMLB[m_] := 
Switch[ArrayDepth[m],
	0,{{m}},
	1,{m},
	_,Transpose[m,Reverse[Table[i,{i,1,ArrayDepth[m]}]]]
	(*TensorRank[m],
	0, {{m}}, 
	1, Transpose[{m}], 
	2, Transpose[m],
	_, Map[Transpose, m, {TensorRank[m] - 2}]*)
]

mShow[___] := engVis[1]

mHide[___] := engVis[0]

(* Internal function, do not call from outside. Does not check if Matlab is started. *)
mClearLastError[]/;!engIsOpen[] := (engOpen[]; mClearLastError[])
mClearLastError[]/;engIsOpen[] := engCmd["lasterror('reset');"];

(* Internal function, do not call from outside. Does not check if Matlab is started. *)
(* This is not thread safe! *)
mGetLastError[]/;!engIsOpen[] := (engOpen[]; mGetLastError[])
mGetLastError[]/;engIsOpen[] := Module[{result,msg,id,n,line},
	engCmd["mConnect_lasterror = lasterror; "<>
           "mConnect_lasterror_N = numel(mConnect_lasterror.stack);"];
	msg = mGetString["mConnect_lasterror.message"];
	id = mGetString["mConnect_lasterror.identifier"];
	n = Round[engGet["mConnect_lasterror_N"][[1,1]]];
	
	If[(msg == "") && (id == "") && (n < 1),

		(* no error *)
		result = "",

		(* error *)
		result = "??? " <> msg <> " [" <> id <> "]\n" <>
		StringJoin[
			Table[
				With[{elem = "mConnect_lasterror.stack(" <> ToString[i] <> ")"},
					"in line " <> mGetString["num2str("<>elem<>".line)"] <>
					" of " <> mGetString[elem<>".name"] <>
					" (" <> mGetString[elem<>".file"] <> ")\n" 
				],
			{i,1,n}]
		
		]
	];

	result
]

(* Not thread-safe. *)
Options[mCmd] = {Debug -> False, DebugFile->"debug"};
mCmd[cmd__String,OptionsPattern[]]/;engIsOpen[] := Module[{result,errors,cmdstring,fn},
	(*Print["Complete command: "<>StringJoin[cmd]];*)
	cmdstring = StringJoin[cmd];
	If[OptionValue[Debug],
		(* Debug - save workspace and write out Matlab code *)
		fn = "['"<>mToMLBString[OptionValue[DebugFile]<>"_data.mat"]<>"']";
		mCmd["save("<>fn<>");"];
		Export[OptionValue[DebugFile]<>"_code.m","load("<>fn<>");\n"<>cmdstring,"Text"];
	];

	(* No Debug - execute command directly *)

	mClearLastError[];
	result = engCmd[cmdstring];
	errors = mGetLastError[];

	If[errors =!= "",
		(Message[mCmd::matlaberror,errors,result];
		Print[errors]; (* We have to output these using Print[], as Message () trims down long messages using '...' *)
		Message[mCmd::matlaberroroutput];
		Print[result];
		$Failed),
		result
	]
		(*
	errors = StringCases[result, RegularExpression["\\?\\?\\?(.*?)\\n"]];
	If[Length[errors]>0,
		(Message[mCmd::matlaberror,StringJoin @@ errors,result];
		$Failed),
		result
	]*)
]

mCmd[cmd__String]/;!engIsOpen[] := (engOpen[]; mCmd[cmd])

mCmd[left___,mVar[var_],right___]/;engIsOpen[] := Module[{id,result},
	id = SymbolName[Unique["globalmConnect"]];
	mPut[id,var];
	result = mCmd[left,id,right];
	mCmd["clear " <> id <> ";"];
	result
]
mCmd[left___,var_?(ArrayQ[#,_,NumberQ]&),right___] := mCmd[left,mVar[var],right]; (* simplifies usage *)
mCmd[left___,var_?NumberQ,right___] := mCmd[left,mVar[var],right]; (* simplifies usage *)

mCmd[left___,mVar[var_],right___]/;!engIsOpen[] := (engOpen[]; mCmd[left,mVar[var],right])

mCmd[___] := Message[mCmd::arg]

SqueezeTrailing[x_?ArrayQ] := NestWhile[
	Function[m,Map[#[[1]]&,m,{-2}]],
	x,
	Dimensions[#]!={}&&Last[Dimensions[#]]==1&
]


mGetCmdList[n_Integer,expr__] /; (n>=1):= Module[{vars,varslist,r},
vars = Table[SymbolName[Unique["globalmConnect"]],{n}];
varslist = StringJoin @@Drop[Flatten[{#,","}&/@vars],-1];
mCmd["["<>varslist<>"] = ",expr];
(r = mGet[#]; mCmd["clear(",mVar[#],");"]; r)& /@ vars
];
mGetCmdList[0,expr__] := (mCmd[expr];);
mGetCmdList[___] := Message[mGetCmdList::arg];

mGetCmd[expr__] := mGetCmdList[1,expr][[1]];
mGetCmd[___] := Message[mGetCmd::arg];


mPut[s_String, x_]/;engIsOpen[] :=
Module[
	{xmlb, dimlab, re, im},
	Switch[Head[x],
		String, mPutString[s,x],
		_, 
			xmlb = ToMLB[x];
			dimlab = Reverse[Dimensions[xmlb]];
			If[
				MemberQ[x, _Complex, {TensorRank[x]}],
				engPut[
					s,
					dimlab,
					N[Re /@ Flatten[xmlb]],
					N[Im /@ Flatten[xmlb]]
				],
				engPut[
					s, 
					dimlab,
					N[Flatten[xmlb]]
				]
			]
	]
]

mPut[s_String, x_]/;!engIsOpen[] := (engOpen[]; mPut[s,x])

mPut[s_String] := mPut[s,Symbol[s]]

mPut[lst:{sl__String}] := mPut /@ lst; (* automatic evaluation on lists *)
mPut[lst:{sl__List}] := mPut @@@ lst;

mPut[___]:= Message[mPut::arg]

Attributes[mPut]:= {}


(* converts special characters, like \[Transpose], to their textual representation: \\[Transpose] *)
mToMLBString[s_String] := StringReplace[StringDrop[StringDrop[ToString[FullForm[s]],{-1}],{1}],{"'"->"''", "\\\\"->"' char(92) '","\\["->"' char(92) '[","\\n"->"' char(10) '","\\t"->"' char(9) '"}]
(* by default, \\[...] sequences in MATLAB output will NOT be interpreted as special characters *)
mFromMLBString[s_String] := s;
(* apply mInterpretString to the result of mGet to substitute special character sequences \\[...] with their corresponding characters *)
mInterpretString[s_String] := ToExpression["\""<>StringReplace[s,{"\\["->"\\[","\\"->"\\\\"}]<>"\""];


mGetString[id_] := StringDrop[mFromMLBString[engCmd["disp(["<>id<>" '.']);"]],-2]
mPutString[id_,value_] :=
	If[mCmd[id <> " = ['" <> mToMLBString[value] <> "'];"] === $Failed,
	$Failed,
	id];				


(* returns class of variable in MATLAB workspace *)
mGetClass[s_String] := StringDrop[mCmd["disp(class("<>s<>"));"],-1]


mGet[s_String]:=
Module[
	{x, ret},
	Switch[mGetClass[s],
		$Failed, $Failed,
		"char", mGetString[s],
		_, x = engGet[s];
		   If[x === $Failed,
			  $Failed,
			  SqueezeTrailing[Transpose[x,Reverse[Range[ArrayDepth[x]]]]]
		   ]
    ]
]

mGet[___] := Message[mGet::arg]

Attributes[mGet] := {Listable}

End[ ]

EndPackage[ ]
